Package: mCSEA
Type: Package
Title: Methylated CpGs Set Enrichment Analysis
Version: 1.32.0
Author: Jordi Martorell-Marugán and Pedro Carmona-Sáez
Maintainer: Jordi Martorell-Marugán <jmartorellm@gmail.com>
Description: Identification of diferentially methylated regions (DMRs) in predefined regions
    (promoters, CpG islands...) from the human genome using Illumina's 450K or EPIC microarray data.
    Provides methods to rank CpG probes based on linear models and includes plotting functions.
Depends: R (>= 3.5), mCSEAdata, Homo.sapiens
Suggests: Biobase, BiocGenerics, BiocStyle, FlowSorted.Blood.450k,
        knitr, leukemiasEset, minfi, minfiData, rmarkdown, RUnit
Imports: biomaRt, fgsea, GenomicFeatures, GenomicRanges, ggplot2,
        graphics, grDevices, Gviz, IRanges, limma, methods, parallel,
        S4Vectors, stats, SummarizedExperiment, utils
VignetteBuilder: knitr
biocViews: ImmunoOncology, DifferentialMethylation, DNAMethylation,
        Epigenetics, Genetics, GenomeAnnotation, MethylationArray,
        Microarray, MultipleComparison, TwoChannel
License: GPL-2
Encoding: UTF-8
LazyData: true
git_url: https://git.bioconductor.org/packages/mCSEA
git_branch: RELEASE_3_23
git_last_commit: 05a0cc5
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-06-03
NeedsCompilation: no
Packaged: 2026-06-04 01:02:48 UTC; biocbuild
Built: R 4.6.0; ; 2026-06-04 19:04:24 UTC; unix
