export(mCSEAPlot)
export(mCSEAPlotGSEA)
export(mCSEATest)
export(rankProbes)
export(mCSEAIntegrate)
import(mCSEAdata)
import(Homo.sapiens)
importFrom("grDevices", "dev.off", "pdf")
importFrom("stats", "cor.test", "lm", "model.matrix",
            "na.omit", "p.adjust", "relevel", "sd")
importFrom("graphics", "abline", "legend", "par", "plot")
importFrom("methods", "is")


